﻿using Kivii.Auth;
using Kivii.Finances.Entities;
using Kivii.Finances.Seeyon.Entities;
using Kivii.Seeyon.Entities;
using Kivii.Text;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Seeyon.Extensions
{
    internal static class ApplyExtension
    {
        public static ResponseToken RequestRestToken(this JsonServiceClient client, string loginName)
        {
            if (client == null) throw new ArgumentNullException("client");
            if (loginName.IsNullOrEmpty()) throw new ArgumentNullException("loginName");

            try
            {
                var request = new RequestRestToken();
                request.userName = Configs.restUserName;
                request.password = Configs.restPassword;
                request.loginName = loginName.ToLower();
                var rtns = client.Post<ResponseToken>($"{Configs.RouteRestToken}", request);
                return rtns;
            }
            catch (Exception ex)
            {
                throw new Exception($"{ex.Message}，可能原因：当前登录人{loginName}，无法与OA账号对应，请联系管理员确认！");
            }
        }

        public static Form_InvoiceApply ConvertToForm(this InvoiceApply apply)
        {
            var bizKvids = new List<string>();
            bizKvids.Add(apply.Kvid.ToString());
            var rtns = new Form_InvoiceApply();
            //var settlementType = "code|MonthlySettlement";//默认月结
            rtns.业务类型 = typeof(BillApply).FullName;
            rtns.业务编号 = bizKvids;
            rtns.编号 = apply.SerialNumber;
            rtns.借票日期 = DateTime.Now.ToString("yyyy-MM-dd");
            rtns.借票科室 = KiviiContext.CurrentMember.DepartmentName;
            rtns.借票人 = KiviiContext.CurrentMember.FullName;
            rtns.借票总额 = apply.Amount.ToString();
            return rtns;
        }

        public static List<Form_InvoiceApplyDetail> ConvertToFormDetail(this InvoiceApply apply)
        {
            var rtns = new List<Form_InvoiceApplyDetail>();

            var index = 1;
            var detail = new Form_InvoiceApplyDetail();
            detail.序号1 = index != -1 ? index.ToString() : "0";
            detail.单位名称 = apply.PayerName;
            detail.金额 = apply.Amount.ToString();
            rtns.Add(detail);
            return rtns;
        }

        public static Form_InvoiceApply ConvertToForm(this List<InvoiceApply> applys)
        {
            var bizKvids = applys.ConvertAll(o => o.Kvid.ToString());
            //var strBizKvids = JsonSerializer.SerializeToString(bizKvids);
            var amount = applys.Sum(o => o.Amount);
            var rtns = new Form_InvoiceApply();
            //var settlementType = "code|MonthlySettlement";//默认月结
            rtns.业务类型 = typeof(BillApply).FullName;
            rtns.业务编号 = bizKvids;
            //rtns.编号 = applys.SerialNumber;
            rtns.借票日期 = DateTime.Now.ToString("yyyy-MM-dd");
            rtns.借票科室 = KiviiContext.CurrentMember.DepartmentName;
            rtns.借票人 = KiviiContext.CurrentMember.FullName;
            rtns.借票总额 = amount.ToString();
            rtns.借票总额人民币 = amount.ConvertToChinese();
            return rtns;
        }

        public static List<Form_InvoiceApplyDetail> ConvertToFormDetail(this List<InvoiceApply> applys)
        {
            var rtns = new List<Form_InvoiceApplyDetail>();

            foreach (var item in applys)
            {
                var index = applys.FindIndex(o => o.Kvid == item.Kvid);
                var detail = new Form_InvoiceApplyDetail();
                detail.序号1 = index != -1 ? index.ToString() : "0";
                detail.单位名称 = item.PayerName;
                detail.金额 = item.Amount.ToString();
                rtns.Add(detail);
            }
            return rtns;
        }

        public static ResponseForm<ResponseData> RequestBillForm(this JsonServiceClient client, Form_InvoiceApply form, List<Form_InvoiceApplyDetail> details, IAuthSession session)
        {
            if (client == null) throw new ArgumentNullException("client");
            if (form == null) throw new ArgumentNullException("form");
            var token = client.RequestRestToken(session.UserAuthName);
            var request = new RequestForm<RequestForm_InvoiceApply>();
            request.appName = Configs.appName;//"collaboration";
            request.data = new RequestData<RequestForm_InvoiceApply>();
            request.data.templateCode = Configs.templateCode;//"JYHTSQ";
            request.data.draft = "0";
            //request.data.senderLoginName = token.bindingUser.loginName;
            request.data.subject = $"预借发票申请({token.bindingUser.name} {DateTime.Now:yyyy-MM-dd HH:mm})";
            request.data.data = new RequestForm_InvoiceApply();
            request.data.data.formmain_0171 = new Form_InvoiceApply();
            request.data.data.formmain_0171 = form;
            request.data.data.formson_0172 = new List<Form_InvoiceApplyDetail>();
            request.data.data.formson_0172 = details;
            Console.WriteLine($"----------------------Route:{Configs.RouteRequestForm}?token={token.id}----------------------------------------");
            Console.WriteLine($"----------------------Request----------------------------------------");
            Console.WriteLine($"{JsonSerializer.SerializeToString(request)}");
            Console.WriteLine($"----------------------End----------------------------------------");
            var rtns = client.Post<ResponseForm<ResponseData>>($"{Configs.RouteRequestForm}?token={token.id}", request);
            return rtns;
        }

        public static string ConvertToChinese(this decimal Num)
        {
            string[] DX_SZ = { "零", "壹", "贰", "叁", "肆", "伍", "陆", "柒", "捌", "玖", "拾" };//大写数字  
            string[] DX_DW = { "元", "拾", "佰", "仟", "万", "拾", "佰", "仟", "亿", "拾", "佰", "仟", "万" };
            string[] DX_XSDS = { "角", "分" };//大些小数单位  
            if (Num == 0) return DX_SZ[0];

            Boolean IsXS_bool = false;//是否小数  

            string NumStr;//整个数字字符串  
            string NumStr_Zs;//整数部分  
            string NumSr_Xs = "";//小数部分  
            string NumStr_R = "";//返回的字符串  


            NumStr = Num.ToString();
            NumStr_Zs = NumStr;
            if (NumStr_Zs.Contains("."))
            {
                NumStr = Math.Round(Num, 2).ToString();
                NumStr_Zs = NumStr.Substring(0, NumStr.IndexOf("."));
                NumSr_Xs = NumStr.Substring((NumStr.IndexOf(".") + 1), (NumStr.Length - NumStr.IndexOf(".") - 1));
                IsXS_bool = true;
            }

            int k = 0;
            Boolean IsZeor = false;//整数中间连续0的情况  
            for (int i = 0; i < NumStr_Zs.Length; i++) //整数  
            {
                int j = int.Parse(NumStr_Zs.Substring(i, 1));
                if (j != 0)
                {
                    NumStr_R += DX_SZ[j] + DX_DW[NumStr_Zs.Length - i - 1];
                    IsZeor = false; //没有连续0  
                }
                else if (j == 0)
                {
                    k++;
                    if (!IsZeor && !(NumStr_Zs.Length == i + 1)) //等于0不是最后一位，连续0取一次  
                    {
                        //有问题  
                        if (NumStr_Zs.Length - i - 1 >= 4 && NumStr_Zs.Length - i - 1 <= 6)
                            NumStr_R += DX_DW[4] + "零";
                        else
                            if (NumStr_Zs.Length - i - 1 > 7)
                            NumStr_R += DX_DW[8] + "零";
                        else
                            NumStr_R += "零";

                        IsZeor = true;
                    }

                    if (NumStr_Zs.Length == i + 1)//  等于0且是最后一位 变成 XX元整  
                        NumStr_R += DX_DW[NumStr_Zs.Length - i - 1];
                }

            }
            if (NumStr_Zs.Length > 2 && k == NumStr_Zs.Length - 1)
                NumStr_R = NumStr_R.Remove(NumStr_R.IndexOf('零'), 1); //比如1000，10000元整的情况下 去0  

            if (!IsXS_bool) return NumStr_R + "整"; //如果没有小数就返回  
            else
            {
                for (int i = 0; i < NumSr_Xs.Length; i++)
                {
                    int j = int.Parse(NumSr_Xs.Substring(i, 1));
                    NumStr_R += DX_SZ[j] + DX_XSDS[NumSr_Xs.Length - i - 1];
                }
            }

            return NumStr_R;
        }
    }
}
